#
# Copyright 3D Control Systems, Inc. All Rights Reserved 2017-2019. Built in San Francisco.
#
# This software is distributed under commercial non-GPL license for personal, educational,
# corporate or any other use. The software as a whole or any parts of that are prohibited
# for distribution and/or use without obtaining license from 3D Control Systems, Inc.
#
# If you do not have the license to use this software, please delete all software files
# immediately and contact sales to obtain the license: sales@3dprinteros.com.
# If you are unsure about the licensing please contact directly our sales: sales@3dprinteros.com.

import logging
import sys
import httplib
import time
import ssl

import config as client_config

config = client_config.get_settings().get('esocket-printrun')
if not config:
    raise Exception('Empty config')


def run(app):
    try:
        esocket = ESocketPrintrun.instance()
        esocket.init()
        import shutil
        import os
        plugin_dir = os.path.dirname(os.path.abspath(__file__))
        shutil.copy(os.path.join(plugin_dir, 'updates/printrun_sender.py'), os.path.join(plugin_dir, '../..'))
    except:
        return False
    esocket.logger.info('ESocket ready')
    esocket.toggle(True)


class ESocketPrintrun(client_config.Singleton):
    MAXTRIES = 5

    def init(self):
        try:
            _create_unverified_https_context = ssl._create_unverified_context
        except AttributeError:
            # Legacy Python that doesn't verify HTTPS certificates by default
            pass
        else:
            # Handle target environment that doesn't support HTTPS verification
            ssl._create_default_https_context = _create_unverified_https_context
        self.logger = logging.getLogger(__name__)
        self.parent = client_config.get_app()
        self.url = config.get('url')
        self.port = config.get('port')
        self.passcode = config.get('passcode')
        self.timeout = config.get('timeout')
        self.esocket_id = config.get('esocket_id')
        self.connection = None

    def connect(self):
        self.logger.debug("{ Connecting...")
        tries = 0
        while not self.parent.stop_flag and tries < self.MAXTRIES:
            try:
                connection = httplib.HTTPSConnection(self.url, port=self.port, timeout=self.timeout)
                connection.connect()
            except Exception as e:
                self.logger.error("Error during HTTP connection: " + str(e))
                self.logger.debug("...failed }")
                self.logger.warning("Warning: connection to %s failed." % self.url)
                tries += 1
                time.sleep(0.2)
            else:
                self.logger.debug("...success }")
                return connection

    def request(self, path, payload='', headers=None):
        self.logger.debug("{ Requesting...")
        if headers is None:
            headers = {
                "Content-Type": "application/json", "Content-Length": str(len(payload)),
                "Connection": "keep-alive"
            }
        try:
            self.connection.request("GET", path, payload, headers)
            resp = self.connection.getresponse()
        except Exception as e:
            self.logger.error("Error during HTTP request:" + str(e))
            time.sleep(0.2)
        else:
            try:
                received = resp.read()
            except Exception as e:
                self.logger.error("Error reading response: " + str(e))
            else:
                if resp.status == httplib.OK and resp.reason == "OK":
                    self.logger.debug("...success }")
                    return received
                else:
                    message = "Error: server responded with non 200 OK\nFull response: %s" % received
                    self.logger.error(message)
        self.logger.debug("...failed }")
        self.logger.warning("Warning: HTTP request failed!")

    def toggle(self, on):
        self.logger.debug('toggle: '+str(on))
        tries = 0
        while not self.parent.stop_flag and tries < self.MAXTRIES:
            if not self.connection:
                self.connection = self.connect()
                if not self.connection:
                    break
            url = "/esocket/%s/%d/?%s" % ("on" if on else "off", self.esocket_id, self.passcode)
            self.logger.debug('url: '+url)
            answer = self.request(url, '')
            self.logger.debug('answer: '+str(answer))
            if not answer:
                tries += 1
                self.connection.close()
                self.connection = None
            else:
                return answer

