#
# Copyright 3D Control Systems, Inc. All Rights Reserved 2017-2019. Built in San Francisco.
#
# This software is distributed under commercial non-GPL license for personal, educational,
# corporate or any other use. The software as a whole or any parts of that are prohibited
# for distribution and/or use without obtaining license from 3D Control Systems, Inc.
#
# If you do not have the license to use this software, please delete all software files
# immediately and contact sales to obtain the license: sales@3dprinteros.com.
# If you are unsure about the licensing please contact directly our sales: sales@3dprinteros.com.

import threaded_sender

# esocket-printrun plugin: START of code update
import time
import threading

try:
    import esocket_printrun
except:
    pass
# esocket-printrun plugin: END of code update


class Sender(threaded_sender.Sender):

    def __init__(self, parent, usb_info, profile):
        self.time_last_restart = None
        threaded_sender.Sender.__init__(self, parent, usb_info, profile)

    # esocket-printrun plugin: START of code update
    # def load_gcodes(self, gcodes):
    #     self.cancel(restart=True)
    #     return threaded_sender.Sender.load_gcodes(self, gcodes)

    def cancel(self, restart=False):
        self.need_restart = False
        try:
            esocket = esocket_printrun.ESocketPrintrun.instance()
        except:
            if restart:
                return None
            return threaded_sender.Sender.cancel(self)
        # some wait for job finished results send
        if restart:
            time.sleep(0.5)
        self.stop_flag = True
        with self.thread_start_lock:
            if self.print_thread and self.print_thread.is_alive():
                self.print_thread.cancel()
                self.print_thread.join()
        self.read_thread.join()
        self.temperature_request_thread.join()
        # self.reset()
        try:
            self.parent.forced_state = "error"
            if self.connection:
                self.connection.close()
            if esocket.toggle(False):
                time.sleep(10)
                if esocket.toggle(True):
                    time.sleep(5)
            else:
                self.reset()
        except:
            pass
        self.stop_flag = False
        operation = 'restart' if restart else 'cancel'
        try:
            self.init()
            self.logger.info("Successful "+operation)
        except RuntimeError:
            self.parent.register_error(255, "Can't reconnect to printer after %s." % operation, is_blocking=True)

    def restart(self):
        return self.cancel(restart=True)

    def is_operational(self):
        # if self.need_restart:
        #     restart_thread = threading.Thread(target=self.restart, name="Restart thread")
        #     restart_thread.start()
        return self.operational_flag

    # esocket-printrun plugin: END of code update
